<?php

namespace Drupal\pagedesigner\Plugin\pagedesigner\Handler;

use Drupal\pagedesigner\Entity\Element;
use Drupal\pagedesigner\Plugin\HandlerPluginBase;

/**
 * Process entities of type "container".
 *
 * @PagedesignerHandler(
 *   id = "container",
 *   name = @Translation("Container render"),
 *   types = {
 *     "container",
 *   },
 * )
 */
class Container extends HandlerPluginBase {

  /**
   * {@inheritdoc}
   */
  public function renderForPublic(Element $entity, array &$build) {
    $entity = $entity->loadNewestPublished();
    if ($entity != NULL) {
      $listBuild = [];
      $i = 0;
      foreach ($entity->children as $item) {
        if ($item->entity != NULL) {
          $listBuild[] = $this->elementHandler->renderForPublic($item->entity) + ['#weight' => $i];
          $i++;
        }
      }
      $listBuild['#prefix'] = '<section class="pd-content pd-live container-fluid">';
      $listBuild['#suffix'] = '</section>';
      $listBuild['#sorted'] = TRUE;
      $build = $listBuild + $build;
      $renderer = \Drupal::service('renderer');
      $renderer->addCacheableDependency($build, $entity);
    }
  }

  /**
   * {@inheritdoc}
   */
  public function render(Element $entity, array &$build = []) {
    $listBuild = [];
    $i = 0;
    foreach ($entity->children as $item) {
      if ($item->entity != NULL) {
        $listBuild[] = $this->elementHandler->render($item->entity) + ['#weight' => $i];
        $i++;
      }
    }
    $listBuild['#prefix'] = '<section class="pd-content pd-live container-fluid">';
    $listBuild['#suffix'] = '</section>';
    $listBuild['#sorted'] = TRUE;
    $build = $listBuild + $build;

    $renderer = \Drupal::service('renderer');
    $renderer->addCacheableDependency($build, $entity);
  }

  /**
   * {@inheritdoc}
   */
  public function renderForEdit(Element $entity, array &$build = []) {
    $listBuild = [];
    $i = 0;
    foreach ($entity->children as $item) {
      if ($item->entity != NULL) {
        $listBuild[] = $this->elementHandler->renderForEdit($item->entity) + ['#weight' => $i];
        $i++;
      }
    }
    $listBuild['#prefix'] = '<section data-grapes-block="content" data-gjs-draggable="false" data-gjs-droppable="true" data-entity-id="' . $entity->id() . '" data-gjs-type="container" class="pd-content pd-edit container-fluid">';
    $listBuild['#suffix'] = '</section>';
    $listBuild['#sorted'] = TRUE;
    $build = $listBuild + $build;

    $renderer = \Drupal::service('renderer');
    $renderer->addCacheableDependency($build, $entity);
  }

  /**
   * {@inheritdoc}
   */
  public function getContent(Element $entity, array &$list = [], $published = TRUE) {
    foreach ($entity->children as $item) {
      if ($item->entity != NULL) {
        $child = ($published) ? $item->entity->loadNewestPublished() : $item->entity;
        if ($child != NULL) {
          $this->elementHandler->getContent($child, $list, $published);
        }
      }
    }
  }

  /**
   * {@inheritdoc}
   */
  public function serialize(Element $entity, array &$result = []) {
    $data = [];
    foreach ($entity->children as $item) {
      $data[] = $item->entity->id();
    }
    $result = $data + $result;
  }

  /**
   * {@inheritdoc}
   */
  public function patch(Element $entity, array $data) {
    if (isset($data['order'])) {
      $entity->children->setValue([]);
      foreach ($data['order'] as $target_id) {
        $entity->children->appendItem($target_id);
      }
      $entity->saveEdit();
    }
  }

}
