<?php

namespace Drupal\pagedesigner\Service;

use Drupal\Core\Entity\ContentEntityBase;
use Drupal\pagedesigner\Entity\Element;
use Drupal\pagedesigner\PagedesignerService;

/**
 * Applies changes of state to entity and pagedesigner content.
 */
class StateChanger extends PagedesignerService {

  /**
   * Publish an entity.
   *
   *   The entity to publish.
   *
   * @return self
   *   Returns itself for chaining.
   */
  public function publish(ContentEntityBase $entity = NULL) {
    if ($entity == NULL) {
      return $this;
    }
    $fields = $this->getPagedesignerFields($entity);
    foreach ($fields as $field) {
      $container = $this->getContainer($entity, $field->getName());
      $this->output = $this->getHandler()->publish($container);
    }
    return $this;
  }

  /**
   * Unpublish an entity.
   *
   *   The entity to unpublish.
   *
   * @return self
   *   Returns itself for chaining.
   */
  public function unpublish(ContentEntityBase $entity = NULL) {
    if ($entity == NULL) {
      return $this;
    }
    $fields = $this->getPagedesignerFields($entity);
    foreach ($fields as $field) {
      $container = $this->getContainer($entity, $field->getName());
      $this->output = $this->getHandler()->unpublish($container);
    }
    return $this;
  }

  /**
   * Copy the pagedesigner content of an entity.
   *
   *   The element to copy.
   *   (Optional) The container for the copied entity.
   *   (Optional) The parent for the copied entity.
   *
   * @return self
   *   Returns itself for chaining.
   */
  public function copy(Element $entity, Element $container = NULL, Element $parent = NULL) {
    if ($entity == NULL) {
      return $this;
    }
    $clone = $this->getHandler()->copy($entity, $container);
    if (!empty($parent)) {
      $clone->parent->entity = $parent;
      $clone->save();
    }
    $this->output = $clone;
    return $this;
  }

  /**
   * Generate a container for an entity.
   *
   * @todo Check if this is actually used.
   *
   *   The entity to generate the container for.
   *
   * @return self
   *   Returns itself for chaining.
   */
  public function generate(ContentEntityBase $entity = NULL) {
    if ($entity == NULL) {
      return $this;
    }
    $this->output = $this->getContainer($entity)->id();
    return $this;
  }

  /**
   * Delete an entity and it's pagedesigner content.
   *
   *   The entity to delete.
   *
   * @return self
   *   Returns itself for chaining.
   */
  public function delete(ContentEntityBase $entity = NULL) {
    if ($entity == NULL) {
      return $this;
    }
    $fields = $this->getPagedesignerFields($entity);
    foreach ($fields as $field) {
      $container = $this->getContainer($entity, $field->getName());
      $this->output = $this->getHandler()->delete($container);
    }
    return $this;
  }

}
